/*
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
| File Name			  : MicroBolt_I2C_Slave.c																					
| Author			    : Micromint, Inc.						
| Copyright       : Copyright  2005, Micromint, Inc.																	
| Creation Date		: 10/9/05																							
| Version			    : 1.00																																												
| Spaces per tab	:	2																									
| Description		  : Main C file																																									 
| Revision			  :	Initial																																																						
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
*/

/*
|----------------------------------------------------
|     Includes
|----------------------------------------------------
*/

#include <ARM/philips/lpc210x.h>
#include <arm_macros.h>
#include "MicroBolt_I2C_Slave.h"

static unsigned char I2cBuffer[20];
static unsigned char I2cBufferIndex = 0;	

/*
|------------------------------------------------------------------------------------------
|     Function        :   main
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Main function for system
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

void main(void)
  {
  __DISABLE_INTERRUPT();                        // Disable all interrupts

/*
|-------------------------
| Config MAM 
|-------------------------
*/
  MAM_CR = 0x00;                                // Turn MAM off (default)
  MAM_TIM = 0x04;                               // Set flash timing to 4 clock cycles

  MAM_CR = 0x02;                                // Fully enable the Memory Accleration Module

/*
|-------------------------
| Config PLL and CCLK 
|-------------------------
*/
  SCB_PLLCFG |= 0x23;				 	   		            // Set to 59 MHz (0x03 is multiply value of 4)
  SCB_PLLCON |= 0x01;                           // Enable the PLL 
  SCB_PLLFEED = 0xAA;					   	  	          // Shadow register copy to enable changes
  SCB_PLLFEED = 0x55;                           // in PLLCON and PLLCFG

/*
|-------------------------
| Config PCLK 
|-------------------------
*/
  SCB_VPBDIV = 0;                               // Peripheral clock is 1/4th Processor clock which equals 14.7456 MHz

/*
|-------------------------
| Configure VIC 
|-------------------------
*/
  VICVectAddr0 = (unsigned)pll_isr;             // Assign the PLL lock ISR vector address
  VICVectCntl0 = INTERRUPT_CHANNEL_FOR_PLL;     // Assign the VIC address to the actual interrupt
  VICIntEnable = INTERRUPT_ENABLE_FOR_PLL;      // Enable the interrupt  

	VICVectAddr1 = (unsigned)I2c_ISR;							// Assign the I2C ISR vector address 
  VICVectCntl1 = INTERRUPT_CHANNEL_FOR_I2C;     // Assign the VIC address to the actual interrupt
  VICIntEnable = INTERRUPT_ENABLE_FOR_I2C;      // Enable the interrupt 

/*
|-------------------------
| Config GPIO 
|-------------------------
*/
  PCB_PINSEL0=0x00000050;                       // Setup with ICCARM App builder - MicroBolt_I2C.bcf (I2C)   
  PCB_PINSEL1=0x55400000;                       //                                   (Secondary JTAG pins) 

  GPIO_IODIR |= MICROBOLT_LED;                  // Setup MicroBolt LED as output

  GPIO_IOCLR=0xffffffff;                        // Clear all pins to start with

/*
|-------------------------
| Config I2C 
|-------------------------
*/
  I2C_I2SCLH=100;                               // I2C clock
  I2C_I2SCLL=100;
  I2C_I2ADR= 0x00000014;											  // Address 10 (0x0A << 1)
  I2C_I2CONSET=0x00000044;                      // Setup flags

  __ENABLE_INTERRUPT();                         // Enable all interrupts

/*
|----------------------------------------------------
|     Start of application
|----------------------------------------------------
*/

  while(1)                                      // Do this forever
    {
    }
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   pll_isr
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Once PLL has locked, connect it and use for system clock
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

#pragma interrupt_handler pll_isr

void pll_isr(void)
  {
  SCB_PLLCON |= 0x02;                           // Connect the PLL 
  SCB_PLLFEED = 0xAA;					   	  	          // Shadow register copy to enable changes
  SCB_PLLFEED = 0x55;                           // in PLLCON and PLLCFG
  VICIntEnClear = PLL_CLR;                      // Disable the PLL interrupt, not needed anymore
  VICVectAddr = VIC_ACK;                        // Acknowledge Interrupt
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   I2c_ISR
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   I2C interrupt and command processing
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

#pragma interrupt_handler I2c_ISR

void I2c_ISR(void)
{
	switch(I2C_I2STAT)
			{

//****************************************************
// I2C Slave Receive Service Routines
//****************************************************

//====================================
// Unique I2C Slave address received
//====================================
			case (0x60):
					{																												// Slave Address and Write received
					I2cBufferIndex = 0;																			// Reset index counter
					I2C_I2CONSET |= 0x0000004;
					break;
					}
//------------------------------------
			case (0x80):																								// Previously addressed with unique address, receive data now
					{
					I2cBuffer[I2cBufferIndex] = I2C_I2DAT;									// Grab the I2C data

					switch(I2cBuffer[0])																		// I2cBuffer[0] is the command
							{
							//========================
							// Command A
							//========================
							case (I2C_CMD_A):															      // Command 'A'
									{
				        	GPIO_IOSET = MICROBOLT_LED;	                    // Turn on MicroBolt LED

									I2cBufferIndex++;																// Increment count index
									if (I2cBufferIndex < I2C_A_PACKET_SIZE)		      // Still more data to come for packet?
											{
											I2C_I2CONSET |= 0x0000004;									// More data to come, ACK'em
											}
									else
											{
								 	    GPIO_IOCLR = MICROBOLT_LED;	                // Turn off MicroBolt LED
											I2C_I2CONCLR = 0x0000004;										// Data done NAK'em to move into state 0x88
											I2cBufferIndex = 0;													// Reset count index
											}
									break;
									}
							//========================
							// Command B
							//========================
							case (I2C_CMD_B):                                   // Command 'B'
									{
				        	GPIO_IOSET = MICROBOLT_LED;	                    // Turn on MicroBolt LED

									I2cBufferIndex++;																// Increment count index
									if (I2cBufferIndex < I2C_B_PACKET_SIZE)		      // Still more data to come for packet?
											{
											I2C_I2CONSET |= 0x0000004;									// More data to come, ACK'em
											}
									else
											{
								 	    GPIO_IOCLR = MICROBOLT_LED;	                // Turn off MicroBolt LED
											I2C_I2CONCLR = 0x0000004;										// Data done NAK'em to move into state 0x88
											I2cBufferIndex = 0;													// Reset count index
											}
									break;
									}
							//========================
							// Default
							//========================
							default:
									{
									I2C_I2CONCLR = 0x0000004;												// NAK'em, not a valid command packet, move into state 0x88
									break;
									}
							}				
					break;
					}											
//------------------------------------
			case (0xA0):																									// STOP or repeated START condition occurred
					{
					I2C_I2CONSET |= 0x0000004;																// ACK'em
					break;
					}
//------------------------------------
			case (0x88):																									// Slave Address correct, but don't want the data so NAK'em
					{																													// Keep NAK'ing them
					I2C_I2CONSET |= 0x0000004;
					break;
					}
//=========================================
// General I2C Slave call address received
//=========================================
			case (0x70):
					{																													// General call address (0x00) received
					I2C_I2CONSET |= 0x0000004;
					break;
					}
//------------------------------------
			case (0x90):																									// Previously addressed with general call address, receive data now
					{
					I2C_I2CONSET |= 0x0000004;																// ACK'em
					break;
					}

//****************************************************
// I2C Slave Transmit Service Routines
//****************************************************
			case (0xA8):																									// Slave Address and Read received 
					{
				 	GPIO_IOSET = MICROBOLT_LED;	                              // Turn on MicroBolt LED

					I2cBufferIndex = 0;																				// Reset index counter

          I2cBuffer[0] = 'C';                                       // First byte is command letter 'C'	
					I2C_I2DAT = I2cBuffer[I2cBufferIndex];                    // First byte to be sent to master																	

					I2cBufferIndex++;																					// Increment count index
					I2C_I2CONSET |= 0x0000004;
					break;
					}
//------------------------------------
			case (0xB8):																									// Data transmitted, ACK received 
					{
					if (I2cBufferIndex < I2C_C_PACKET_SIZE)			              // Still more data to send for packet?
							{
							I2C_I2DAT = I2cBuffer[I2cBufferIndex];                // Assign next byte
							I2cBufferIndex++;																			// Increment count index																		

							if (I2cBufferIndex == I2C_C_PACKET_SIZE)
									{
								 	GPIO_IOCLR = MICROBOLT_LED;	                      // Turn off MicroBolt LED
									}
							}
					else
							{																											
							I2C_I2DAT = 0x255;																		// SDA High, no more data to send
							}

					I2C_I2CONSET |= 0x0000004;																// Set AA
					break;
					}
//------------------------------------
			case (0xC0):																									// Data transmitted, NAK received
					{
					I2C_I2DAT = 0x255;																				// SDA High, master doesn't want anymore data
					I2C_I2CONSET |= 0x0000004;																// Set AA
					break;
					}
//------------------------------------
			case (0xC8):																									// Slave Address or general call correct, data same as last transmitted
					{																													// Not a valid state, but ACK'em just in case
					I2C_I2CONSET |= 0x0000004;
					break;
					}
//------------------------------------

//------------------------------------
			default:
					{
					I2C_I2CONSET |= 0x0000004;																// When in doubt ACK'em
					break;
					}
			}
//------------------------------------

	I2C_I2CONCLR = I2C_CLR;																				    // I2C interrupt clear
	VICVectAddr = VIC_ACK; 																						// Acknowledge Interrupt
}